#include <Application.h>
#include <Window.h>
#include <View.h>
#include <File.h>
#include <Message.h>
#include <Path.h>
#include <Roster.h>
#include <Alert.h>
#include <Slider.h>
#include <Button.h>
#include <String.h>
#include <StringView.h>
#include <TranslationUtils.h>
#include <Bitmap.h>
#include <Box.h>
#include <MenuBar.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>
#include <MidiPort.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "app.h"

MidiPort *midi;
BPath MyPath;
EfxData *Efx88Data, *EfxSpData, *SourceData, *Limits;
TPreferences *Prefs;

//Preferences
TPreferences::TPreferences(char *filename) : BMessage('pref') {
	BFile file;
	
	path = MyPath;
	path.Append(filename);
	status = file.SetTo(path.Path(), B_READ_ONLY);
	if(status == B_OK){
		status = Unflatten(&file);
	}
}

TPreferences::~TPreferences() {
	BFile file;
	
	if (file.SetTo(path.Path(), B_WRITE_ONLY | B_CREATE_FILE) == B_OK) {
		Flatten(&file);
	}
}


status_t TPreferences::SetBool(const char *name, bool b) {
	if (HasBool(name)) {
		return ReplaceBool(name, 0, b);
	}
	return AddBool(name, b);
}

status_t TPreferences::SetInt8(const char *name, int8 i) {
	if (HasInt8(name)) {
		return ReplaceInt8(name, 0, i);
	}
	return AddInt8(name, i);
}

status_t TPreferences::SetInt16(const char *name, int16 i) {
	if (HasInt16(name)) {
		return ReplaceInt16(name, 0, i);
	}
	return AddInt16(name, i);
}

status_t TPreferences::SetInt32(const char *name, int32 i) {
	if (HasInt32(name)) {
		return ReplaceInt32(name, 0, i);
	}
	return AddInt32(name, i);
}

status_t TPreferences::SetInt64(const char *name, int64 i) {
	if (HasInt64(name)) {
		return ReplaceInt64(name, 0, i);
	}
	return AddInt64(name, i);
}

status_t TPreferences::SetFloat(const char *name, float f) {
	if (HasFloat(name)) {
		return ReplaceFloat(name, 0, f);
	}
	return AddFloat(name, f);
}

status_t TPreferences::SetDouble(const char *name, double f) {
	if (HasDouble(name)) {
		return ReplaceDouble(name, 0, f);
	}
	return AddDouble(name, f);
}

status_t TPreferences::SetString(const char *name, const char *s) {
	if (HasString(name)) {
		return ReplaceString(name, 0, s);
	}
	return AddString(name, s);
}

status_t TPreferences::SetPoint(const char *name, BPoint p) {
	if (HasPoint(name)) {
		return ReplacePoint(name, 0, p);
	}
	return AddPoint(name, p);
}

status_t TPreferences::SetRect(const char *name, BRect r) {
	if (HasRect(name)) {
		return ReplaceRect(name, 0, r);
	}
	return AddRect(name, r);
}

status_t TPreferences::SetMessage(const char *name, const BMessage *message) {
	if (HasMessage(name)) {
		return ReplaceMessage(name, 0, message);
	}
	return AddMessage(name, message);
}

status_t TPreferences::SetFlat(const char *name, const BFlattenable *obj) {
	if (HasFlat(name, obj)) {
		return ReplaceFlat(name, 0, (BFlattenable *) obj);
	}
	return AddFlat(name, (BFlattenable *) obj);
}

//Effect Data
EfxData::EfxData(BPath *path, const char *filename)
	:BMessage(EFFECT_DATA){
	FILE *file;
	char section[64], key[64], value[64];
	int i=0,j=0;
	path->Append(filename);
	if((file = fopen(path->Path(), "r"))==NULL){
		status = false;
		return;
	}
	//最初の[]まで進める
	while(fgetc(file)!='['){
		if(feof(file))break;
	}
	//ファイルの終端までループ
	while(!feof(file)){
		//[]内の文字を取り込む
		i=0;
		while((section[i++] = fgetc(file)) != ']');
		section[i-1] = '\0';
		//スペースを飛ばす
		while(isspace(key[i] = fgetc(file)));
		//'['だったら次のセクション
		//そうでなかったらキーとバリューを読み取る
		strcpy(key,section);
		key[i-1]='_';
		j=i+1;
		//スペースを飛ばす
		while(isspace(key[j] = fgetc(file)));
		while(key[j] != '[' && !feof(file)){
			while((key[++j] = fgetc(file)) != '=')if(feof(file))break;
			if(feof(file))break;
			key[j]='\0';
			j=0;
			while((value[j] = fgetc(file)) != '\n')j++;
			int k=0;
			while(isspace(value[j-k]))k++;
			value[j-k+1]='\0';
			AddString(key, value);
			j=i;
			//スペースを飛ばす
			while(!feof(file)){
				if(isspace(key[j] = fgetc(file)))continue;
				else{
					break;
				}
			}
		}
	}
	fclose(file);
	status = true;
}

bool EfxData::InitCheck(void){
	return status;
}

//ビューの onDraw 関数
void View::Draw(BRect updateRect){
}

void MidiPort::SetEffect88Pro(int8 data1, int8 data2){
/*	int8 data[14];
	data[0]=0xf0;
	data[1]=0x41;
	data[2]=0x10;
	data[3]=0x00;
	data[4]=0x11;
	data[5]=0x12;
	data[6]=0x00;
	data[7]=0x40;
	data[8]=0x01;
	data[9]=0x00;
	data[10]=data1;
	data[11]=data2;
	data[12]=0x00;
	for(int i=6;i<=11;i++){
		data[12]-=data[i];
	}
	data[13]=0xf7;
*/	int8 data[12];
	int sum=0;
	data[0]=0xf0;
	data[1]=0x41;
	data[2]=0x10;
	data[3]=0x42;
	data[4]=0x12;
	data[5]=0x40;
	data[6]=0x03;
	data[7]=0x00;
	data[8]=data1;
	data[9]=data2;
	for(int i=5;i<=9;i++){
		sum+=data[i];
	}
	sum=128-sum%128;
	data[10]=(int8)sum;
	data[11]=0xf7;
	midi->SystemExclusive((void*)&data, sizeof(data));
}

void MidiPort::SetEffectParameter88Pro(int8 data1, int address){
	int8 data[11];
	int sum=0;
	data[0]=0xf0;
	data[1]=0x41;
	data[2]=0x10;
	data[3]=0x42;
	data[4]=0x12;
	data[5]=0x40;
	data[6]=0x03;
	data[7]=address;
	data[8]=data1;
	for(int i=5;i<=8;i++){
		sum+=data[i];
	}
	sum=128-sum%128;
	data[9]=(int8)sum;
	data[10]=0xf7;
	midi->SystemExclusive((void*)&data, sizeof(data));
}

void MidiPort::SetEffectMode(int8 mode){
	int8 data[14];
	int sum=0;
	data[0]=0xf0;
	data[1]=0x41;
	data[2]=0x10;
	data[3]=0x00;
	data[4]=0x11;
	data[5]=0x12;
	data[6]=0x00;
	data[7]=0x40;
	data[8]=0x40;
	data[9]=0x00;
	data[10]=mode;
	data[11]=0x00;
	for(int i=6;i<=11;i++){
		sum+=data[i];
	}
	sum=128-sum%128;
	data[12]=(int8)sum;
	data[13]=0xf7;
	midi->SystemExclusive((void*)&data, sizeof(data));
}

//ファイルから項目を読み出すメニューのコンストラクタ
Menu::Menu(const char *label, EfxData *efxdata, char *section,
	const uint32 messageid)
	: BMenu(label){
	char key[64], *value, *strtmp;
	int num, i;
	int data1,data2;
	BMessage *message;
	
	strcpy(key,section);
	strcat(key,"_NUM");
	if(efxdata->InitCheck()){
		efxdata->FindString(key, (const char **)&strtmp);
		sscanf(strtmp, "%d", &num);
		
		for(i=0;i<num;i++){
			sprintf(key, "%s_%d", section, i+1);
			efxdata->FindString(key, (const char **)&value);
			
			//MSB,LSB をメッセージに追加
			sprintf(key, "%s_MSB%d", section, i+1);
			message = new BMessage(messageid);
			if(efxdata->FindString(key, (const char **)&strtmp) == B_OK){
				sscanf(strtmp, "%X,%X", &data1, &data2);
				message->AddInt8("MSB", data1);
				message->AddInt8("LSB", data2);
			}
			AddItem(new BMenuItem(value, message));
		}
	}
	else{
		AddItem(new BMenuItem("no file", NULL));
	}
	ItemAt(0)->SetMarked(true);
	SetLabelFromMarked(true);
}

//MenuBar のコンストラクタ
MenuBar::MenuBar(BRect frame) : BMenuBar(frame, "MenuBar"){
	BMenu *menu;
	BMenuItem *menuitem;
	menu = new BMenu("Port");
	int32 num;
	char *port;
	if(Prefs->FindString("port", (const char**)&port) != B_OK){
		port = new char[strlen(DEF_PORT_NAME)];
		strcpy(port, DEF_PORT_NAME);
	}
	char name[256];
	num=midi->CountDevices();
	for(int32 i=0;i<num;i++){
		midi->GetDeviceName(i, name);
		menu->AddItem(new BMenuItem(name, new BMessage(MENU_PORT_CHANGED)));
	}
	menu->SetRadioMode(true);
	menuitem = menu->FindItem(port);
	if(menuitem == NULL){
		if(menu->CountItems()==0){
			BAlert *alert;
			alert = new BAlert("info","no MIDI port detected.","close");
			alert->Go();
			be_app->PostMessage(B_QUIT_REQUESTED);
		}
		else{
			midi->Open(menu->ItemAt((int32)0)->Label());
			menu->ItemAt((int32)0)->SetMarked(true);
		}
	}
	else{
		menuitem->SetMarked(true);
		midi->Open(port);
	}
	AddItem(menu);
}

//MenuField のコンストラクタ
MenuField::MenuField(BRect frame, const char *name, BMenu *menu)
	: BMenuField(frame, name, name, menu, false){
}

//Slider のコンストラクタ
Slider::Slider(BRect frame, char *name, BMessage *message, rgb_color color,
		uint16 min, uint16 max, uint16 defvalue, const char *label, int typetmp)
	: BSlider(frame, name, label, message, min, max, B_VERTICAL,
		B_TRIANGLE_THUMB)
{
	type = typetmp;
	//エフェクトパラメータの場合 LimitLabel をセット
	if(type){
		SetOrientation(B_HORIZONTAL);
		char *MaxLabel,*MinLabel,key[16];
		status_t status1, status2;
		sprintf(key, "%d_minlabel", type);
		status1=Limits->FindString(key, (const char **)&MinLabel);
		sprintf(key, "%d_maxlabel", type);
		status2=Limits->FindString(key, (const char **)&MaxLabel);
		if(status1==B_OK && status2==B_OK)
			SetLimitLabels(MinLabel, MaxLabel);
		else
			SetLimitLabels("", "");
	}
	else{
		SetLimitLabels("", "");
	}
	SetValue(defvalue);
	SetModificationMessage(new BMessage(*message));
	SetHashMarks(B_HASH_MARKS_BOTH);
	SetHashMarkCount(5);
	DrawFocusMark();
	SetKeyIncrementValue(1);
	UseFillColor(true,&color);
	ResizeToPreferred();
}

int Slider::Type(void){
	return type;
}

//スライダーのステータス表示
char* Slider::UpdateText(void) const {
	static char Status[16];
	switch(type){
		//Gain type
		case 2:
		case 75:
		case 76:
		case 80:
			sprintf(Status, "%ddB", (int)Value()-64);
			break;
		
		//Accel type
		case 3:
			sprintf(Status, "%d", (int)Value());
			break;
		
		//2value type
		case 4:
		case 5:
		case 9:
		case 14:
		case 15:
		case 28:
		case 30:
		case 31:
		case 40:
		case 43:
		case 46:
		case 47:
		case 48:
		case 50:
		case 51:
			if(Value())
				strcpy(Status, MaxLimitLabel());
			else
				strcpy(Status, MinLimitLabel());
			break;
		
		//Few elements type
		case 10:
		case 11:
		case 17:
		case 23:
		case 24:
		case 25:
		case 34:
		case 35:
		case 45:
		case 49:
		case 60:
		case 62:
		case 69:
		case 87:
		{
			BString str= MinLimitLabel();
			str.Append(",");
			for(int i=0;i<Value();i++){
				str.Remove(0,str.FindFirst(',')+1);
			}
			str.Remove(str.FindFirst(','),str.CountChars()-str.FindFirst(','));
			strcpy(Status, str.LockBuffer(str.CountChars()));
			break;
		}
		
		//2value devided at center.
		case 16:
		{
			if(Value()<64)strcpy(Status, MinLimitLabel());
			else strcpy(Status, MaxLimitLabel());
			break;
		}
		
		//log type(skip count in 0-7F).See the message loop of EffectWindow.
		case 6:
		{
			char *freq[]={"200Hz","250Hz","315","400","500","630","800","1000",
				"1250","1600","2000","2500","3150","4000","5000","6300"};
			sprintf(Status, "%sHz", freq[Value()]);
			break;
		}
		
		case 18:
		{
			char *freq[]={"250","315","400","500","630","800","1000",
				"1250","1600","2000","2500","3150","4000","5000","6300","8000"};
			sprintf(Status, "%sHz", freq[Value()]);
			break;
		}
		
		case 32:
		{
			char *freq[]={"315Hz","400Hz","500Hz","630Hz",
				"800Hz","1000Hz","1250Hz","1600Hz","2000Hz","2500Hz",
				"3150Hz","4000Hz","5000Hz","6300Hz","8000Hz","Bypass"};
			strcpy(Status, freq[Value()]);
			break;
		}
		
		case 44:
		{
			char *freq[]={"250Hz","315Hz","400Hz","500Hz","630Hz",
				"800Hz","1000Hz","1250Hz","1600Hz","2000Hz","2500Hz",
				"3150Hz","4000Hz","5000Hz","6300Hz","Bypass"};
			strcpy(Status, freq[Value()]);
			break;
		}
		
		case 64:
		{
			int freq[]={20,25,35,50,85,115,150,200,250,350,500,650,850,1000,
				1500,2000};
			sprintf(Status, "%dHz", freq[Value()]);
			break;
		}
		
		//log type2(full count)
		case 12:
		{
			int freq;
			if(Value() <= 20)freq = 100 + Value() * 10;
			else if(Value() <= 55)freq = 300 + (Value()-20) * 20;
			else if(Value() <= 125)freq = 1000 + (Value()-55) * 100;
			else freq = 8000;
			sprintf(Status, "%dHz", freq);
			break;
		}

		case 13:
		{
			float freq;
			if(Value() <= 99)freq = ((float)Value()+1.0) * 0.05;
			else if(Value() <= 119)freq = 5.0 + ((float)Value()-99.0) * 0.1;
			else if(Value() <= 125)freq = 7.0 + ((float)Value()-119.0) * 0.5;
			else freq = 10.0;
			sprintf(Status, "%2.2fHz", freq);
			break;
		}

		case 19:
		{
			float time;
			if(Value() <= 50)time = (float)Value() * 0.1;
			else if(Value() <= 60)time = 5.0 + ((float)Value()-50.0) * 0.5;
			else if(Value() <= 100)time = 10.0 + ((float)Value()-60.0) * 1.0;
			else if(Value() <= 125)time = 50.0 + ((float)Value()-100.0) * 2.0;
			else time = 100.0;
			sprintf(Status, "%2.1fms", time);
			break;
		}
		
		case 29:
		{
			float time;
			if(Value() <= 50)time = (float)Value() * 0.1;
			else if(Value() <= 60)time = 5.0 + ((float)Value()-50.0) * 0.5;
			else if(Value() <= 90)time = 10.0 + ((float)Value()-60.0) * 1.0;
			else if(Value() <= 116)time = 40.0 + ((float)Value()-90.0) * 10.0;
			else if(Value() <= 126)time = 300.0 + ((float)Value()-116.0) * 20.0;
			else time = 500.0;
			sprintf(Status, "%2.1fms", time);
			break;
		}

		case 33:
		{
			int time;
			if(Value() <= 70)time = 200+(int)Value() * 5;
			else time = 550 + ((int)Value()-70) * 10;
			sprintf(Status, "%dms", time);
			break;
		}
		
		case 55:
		{
			int time;
			if(Value() <= 80)time = 200+(int)Value() * 5;
			else if(Value() <= 120)time = 600 + ((int)Value()-80) * 10;
			else time = 1000;
			sprintf(Status, "%dms", time);
			break;
		}
		
		case 65:
		{
			float time;
			if(Value() <= 99)time = ((float)Value()+1.0) * 0.1;
			else time = 10.0 + ((float)Value()-99) * 1.0;
			sprintf(Status, "%2.1fsec", time);
			break;
		}

		case 81:
		{
			int freq;
			if(Value() <= 7)freq = 60 + (int)Value() * 20;
			else freq = 200 + ((int)Value()-7) * 100;
			sprintf(Status, "%dHz", freq);
			break;
		}

		//linear
		case 52:
		{
			sprintf(Status, "%2.2fHz", ((float)Value()+1.0)*0.05);
			break;
		}

		case 53:
		{
			sprintf(Status, "%dms", (int)Value()*5);
			break;
		}

		case 73:
		{
			sprintf(Status, "%d", (int)Value()-10);
			break;
		}

		//percentage
		case 20:
		{
			sprintf(Status, "%d%%", (int)Value()*2-128);
			break;
		}
		
		//cent
		case 37:
		{
			sprintf(Status, "%dcent", (int)Value()*2-128);
			break;
		}
		
		//value*2
		case 21:
		{
			sprintf(Status, "%d", (int)Value()*2);
			break;
		}
		
		//value*5
		case 82:
		{
			sprintf(Status, "%dms", (int)Value()*5);
			break;
		}
		
		//value+1
		case 41:
		case 42:
		case 86:
		{
			sprintf(Status, "%d", (int)Value()+1);
			break;
		}
		
		//Time as distance
		case 56:
		{
			sprintf(Status, "%dm", ((int)Value()+1)*5);
			break;
		}
		
		
		//Center is 64
		case 27:
		case 36:
		case 85:
		{
			sprintf(Status, "%d", (int)Value()-64);
			break;
		}
		
		//log type3(no skip 0-less than 7F)
		case 7:
		{
			char *q[]={"0.5","1.0","2.0","4.0","9.0"};
			strcpy(Status, q[Value()]);
			break;
		}

		//Pan type
		case 8:
			if(Value()<64)
				sprintf(Status, "L%d", 64-(int)Value());
			else if(Value()>64)
				sprintf(Status, "R%d", (int)Value()-64);
			else if(Value()==64)
				strcpy(Status, "center");
			break;
		
		case 22:
			if(Value()<64)
				sprintf(Status, "D100:%dE", (int)((float)Value()*100.0/64.0));
			else if(Value()==64)
				strcpy(Status, "D100:100E");
			else if(Value()>64)
				sprintf(Status, "D%d:100E", (int)((127.0-(float)Value())*100.0/63.0));
			break;
		
		case 54:
			if(Value()<64)
				sprintf(Status, "A100:%dB", (int)((float)Value()*100.0/64.0));
			else if(Value()>64)
				sprintf(Status, "A%d:100B", (int)((127.0-(float)Value())*100.0/63.0));
			else if(Value()==64)
				strcpy(Status, "A100:100B");
			break;
		
		//Azimuth
		case 39:
		{
			if(Value() <= 14)sprintf(Status, "L%d", (15-(int)Value())*12);
			else if(Value() == 15)strcpy(Status, "center");
			else sprintf(Status, "R%d", ((int)Value()-15)*12);
			break;
		}
		
		default:
		{
			sprintf(Status, "%d", (int)Value());
			break;
		}
	}
	return Status;
}

//ビュークラスのコンストラクタ
View::View(BRect frame)
	: BView(
		frame,
		NULL,
		B_FOLLOW_ALL_SIDES,
		B_WILL_DRAW
	){
	//View のカスタマイズ
	SetViewColor(MAIN_VIEW_COLOR);

	//Read Preferences
	bool effectmode;
	if(Prefs->FindBool("effect_mode", &effectmode) != B_OK){
		effectmode = false;
	}

	int8 inputmode;
	if(Prefs->FindInt8("input_mode", &inputmode) != B_OK){
		inputmode = 0;
	}
	
	
	//メニューフィールド作成(Full Effect Select)
	MenuField *FullEffectMenuField, *CompactEffectMenuField;
	BRect MMFrame(SECTION_SIZE_INPUT*2, 21, Bounds().right, 42);
	FullEffectMenuField = new MenuField(
		MMFrame,
		"Effect",
		new Menu("FullEffect", EfxSpData, "TYPENAME", MENU_EFFECT_CHANGED)
		);
	FullEffectMenuField->SetDivider(35);
	if(effectmode)FullEffectMenuField->Hide();
	AddChild(FullEffectMenuField);

	//メニューフィールド作成(Compact Effect Select)
	CompactEffectMenuField = new MenuField(
		MMFrame,
		"Effect",
		new Menu("ConpactEffect", Efx88Data, "TYPENAME", MENU_EFFECT_CHANGED)
		);
	CompactEffectMenuField->SetDivider(35);
	if(!effectmode)CompactEffectMenuField->Hide();
	AddChild(CompactEffectMenuField);
	
	//スライダー作成
	BRect SFrame(SECTION_SPACE/2-1,frame.bottom-127-30-50,
		SECTION_SIZE_INPUT-SECTION_SPACE/2,frame.bottom-50)
		,ButFrame,SourceMenuFrame,LabelFrame,BoxRect;
	char str[16];
	Slider *aSlider=NULL;
	BStringView *LabelView;
	BMessage *message;
	for(uint32 i=0;i<4;i++){
		//EFFECT,SOLO,MUTE Buttons
		ButFrame=SFrame;
		ButFrame.top-=25;
		ButFrame.bottom=SFrame.top-1;
		sprintf(str, "Solo%d", (int)i);
		message = new BMessage(BUTTON_SOLO_CHANGED);
		AddChild(new BButton(ButFrame,str,"solo",message));

		ButFrame.top -= 24;
		ButFrame.bottom -= 24;
		sprintf(str, "Mute%d", (int)i);
		message = new BMessage(BUTTON_MUTE_CHANGED);
		AddChild(new BButton(ButFrame,str,"mute",message));
		
		ButFrame.top -= 24;
		ButFrame.bottom -= 24;
		sprintf(str, "Effect%d", (int)i);
		message = new BMessage(BUTTON_EFFECT_CHANGED);
		message->AddPointer("fulleffect",FullEffectMenuField);
		message->AddPointer("compacteffect",CompactEffectMenuField);
		AddChild(new BButton(ButFrame,str,"effect",message));
		//Slider
		sprintf(str, "Slider%d", (int)i);
		aSlider = new Slider(
			SFrame,
			str,
			new BMessage(INPUT_SLIDER_CHANGED),
			MAIN_SLIDER_COLOR
			);
		AddChild(aSlider);
		//Number Label
		LabelFrame = SFrame;
		LabelFrame.top = SFrame.bottom;
		LabelFrame.bottom = Bounds().bottom;
		sprintf(str,"%d",(int)i+1);
		LabelView = new BStringView(LabelFrame,NULL,str);
		LabelView->SetAlignment(B_ALIGN_CENTER);
		BFont font(be_bold_font);
		font.SetSize(16.0);
		LabelView->SetFont(&font);
		AddChild(LabelView);
		//Name Label
		LabelFrame = aSlider->Bounds();
		LabelFrame.top = LabelFrame.bottom-15;
		sprintf(str,"Label%d",(int)i);
		LabelView = new BStringView(LabelFrame,str,SliderLabel[i]);
		LabelView->SetAlignment(B_ALIGN_CENTER);
		aSlider->AddChild(LabelView);
		SFrame.left+=SECTION_SIZE_INPUT;
		SFrame.right+=SECTION_SIZE_INPUT;
		//Box
		if(i){
			BoxRect.Set(
				SECTION_SIZE_INPUT*i*(i!=1), ButFrame.top-SECTION_SPACE,
				SECTION_SIZE_INPUT*(i+1)+1, Bounds().bottom
				);
			AddChild(new BBox(BoxRect));
		}
	}
	
	if(inputmode != 2){
		//Hide the 2nd set.
		FindView("Effect1")->Hide();
		FindView("Mute1")->Hide();
		FindView("Solo1")->Hide();
		FindView("Slider1")->Hide();
		//１番目のセクションを１・２の中央にずらす				
		FindView("Effect0")->MoveBy(SECTION_SIZE_INPUT/2,0);
		FindView("Mute0")->MoveBy(SECTION_SIZE_INPUT/2,0);
		FindView("Solo0")->MoveBy(SECTION_SIZE_INPUT/2,0);
		FindView("Slider0")->MoveBy(SECTION_SIZE_INPUT/2,0);
	}
	
	//アウトプットセクション
	MenuField *SourceMenu;
	Menu *outmenu;
	SFrame.left = Bounds().right-SECTION_SIZE_OUTPUT*2+SECTION_SPACE/2;
	SFrame.right = SFrame.left+SECTION_SIZE_OUTPUT-SECTION_SPACE/2;
	for(uint32 i=14;i<16;i++){
		SourceMenuFrame=SFrame;
		SourceMenuFrame.bottom=SFrame.top-1;
		SourceMenuFrame.top-=24;
		sprintf(str, "SourceMenu%d", (int)i-14);
		outmenu =new Menu(str,
				SourceData,
				"SOURCE",
				MENU_SOURCE_CHANGED
				);
		if(BMenuItem *item=outmenu->ItemAt((int32)(9*(i==15)))){
			item->SetMarked(true);
		}
		if(i==15)outmenu->AddItem(new BMenuItem("rec",new BMessage(MENU_SOURCE_CHANGED)));
		SourceMenu = new MenuField(SourceMenuFrame,
			NULL,
			outmenu
			);
		AddChild(SourceMenu); 
		sprintf(str, "Slider%d", (int)i);
		aSlider = new Slider(
			SFrame,
			str,
			new BMessage(INPUT_SLIDER_CHANGED),
			MAIN_SLIDER_COLOR
			);
		AddChild(aSlider);
		//Number Label
		LabelFrame = SFrame;
		LabelFrame.top = SFrame.bottom;
		LabelFrame.bottom = Bounds().bottom;
		sprintf(str,"%d",(int)i+1);
		LabelView = new BStringView(LabelFrame,NULL,str);
		LabelView->SetAlignment(B_ALIGN_CENTER);
		BFont font(be_bold_font);
		font.SetSize(16.0);
		LabelView->SetFont(&font);
		AddChild(LabelView);
		//Name Label
		LabelFrame = aSlider->Bounds();
		LabelFrame.top = LabelFrame.bottom-15;
		sprintf(str,"Label%d",(int)i);
		LabelView = new BStringView(LabelFrame,str,SliderLabel[i]);
		LabelView->SetAlignment(B_ALIGN_CENTER);
		aSlider->AddChild(LabelView);
		
		SFrame.left+=SECTION_SIZE_OUTPUT;
		SFrame.right+=SECTION_SIZE_OUTPUT;
	}
	//Box
	BoxRect.Set(
		Bounds().right-SECTION_SIZE_OUTPUT*2-1,
		SourceMenuFrame.top-SECTION_SPACE,
		Bounds().right-SECTION_SIZE_OUTPUT,
		Bounds().bottom
		);
	AddChild(new BBox(BoxRect));
	BoxRect.Set(
		Bounds().right-SECTION_SIZE_OUTPUT-1,
		SourceMenuFrame.top-SECTION_SPACE,
		Bounds().right,
		Bounds().bottom
		);
	AddChild(new BBox(BoxRect));
	
	//メニューフィールド作成(インプットセレクタ)
	MenuField *InputMenu;
	BMenu *inmenu;
	inmenu = new BMenu("Input");
	inmenu->AddItem(new BMenuItem("Mic", new BMessage(MENU_INPUT_CHANGED)));
	inmenu->AddItem(new BMenuItem("Line", new BMessage(MENU_INPUT_CHANGED)));
	inmenu->AddItem(new BMenuItem("StereoMic", new BMessage(MENU_INPUT_CHANGED)));
	inmenu->ItemAt((int32)inputmode)->SetMarked(true);
	inmenu->SetLabelFromMarked(true);
	BRect IMFrame(SECTION_SPACE,SECTION_SPACE,105,40);
	InputMenu = new MenuField(
		IMFrame,
		"Input",
		inmenu
		);
	InputMenu->SetDivider(25);
	AddChild(InputMenu);
	

	//MenuField(Effect Mode)
	MenuField *ModeMenuField;
	BMenu *ModeMenu;
	MMFrame.top=0;
	MMFrame.bottom=21;
	ModeMenu = new BMenu("Mode");

	message = new BMessage(MENU_MODE_CHANGED);
	message->AddPointer("MenuShow", FullEffectMenuField);
	message->AddPointer("MenuHide", CompactEffectMenuField);
	ModeMenu->AddItem(new BMenuItem("Full Effect", message));

	message = new BMessage(MENU_MODE_CHANGED);
	message->AddPointer("MenuHide", FullEffectMenuField);
	message->AddPointer("MenuShow", CompactEffectMenuField);
	ModeMenu->AddItem(new BMenuItem("Compact Effect", message));

	ModeMenu->SetLabelFromMarked(true);
	ModeMenu->ItemAt((int32)effectmode)->SetMarked(true);
	ModeMenuField = new MenuField(
		MMFrame,
		"Mode",
		ModeMenu
		);
	ModeMenuField->SetDivider(35);
	AddChild(ModeMenuField);
	
	//Send MIDI Message
	midi->SetEffectMode(4-effectmode);
	midi->ControlChange(1, 21, inputmode);
}

//Message loop
void Window::MessageReceived(BMessage *message){
	BButton *button;
	BMenuItem *item;
	char name[256],str[256];
	int32 index;
	int i;
	switch(message->what){
		case INPUT_SLIDER_CHANGED:
		{
			BSlider* slider;
			message->FindPointer("source", (void **) &slider);
			strcpy(name,slider->Name());
			for(i=0;i<16;i++){
				sprintf(str,"Slider%d",i);
				if(!strcmp(str,name)){
					midi->ControlChange(i+1, 7, slider->Value());
					break;
				}
			}
			break;
		}
		case MENU_PORT_CHANGED:
		{
			message->FindInt32("index", &index);
			midi->Close();
			midi->GetDeviceName(index, name);
			midi->Open(name);
			break;
		}
		case MENU_INPUT_CHANGED:
		{
			message->FindInt32("index", &index);
			midi->ControlChange(1, 21, index);
			if(index != 2){
				if(!FindView("Effect1")->IsHidden()){
					FindView("Effect1")->Hide();
					FindView("Mute1")->Hide();
					FindView("Solo1")->Hide();
					FindView("Slider1")->Hide();
					
					FindView("Effect0")->MoveBy(SECTION_SIZE_INPUT/2,0);
					FindView("Mute0")->MoveBy(SECTION_SIZE_INPUT/2,0);
					FindView("Solo0")->MoveBy(SECTION_SIZE_INPUT/2,0);
					FindView("Slider0")->MoveBy(SECTION_SIZE_INPUT/2,0);
				}
			}
			else{
				if(FindView("Effect1")->IsHidden()){
					FindView("Effect1")->Show();
					FindView("Mute1")->Show();
					FindView("Solo1")->Show();
					FindView("Slider1")->Show();
					
					FindView("Effect0")->MoveBy(-SECTION_SIZE_INPUT/2,0);
					FindView("Mute0")->MoveBy(-SECTION_SIZE_INPUT/2,0);
					FindView("Solo0")->MoveBy(-SECTION_SIZE_INPUT/2,0);
					FindView("Slider0")->MoveBy(-SECTION_SIZE_INPUT/2,0);
				}
			}
			break;
		}
		case MENU_SOURCE_CHANGED:
		{
			message->FindPointer("source", (void **) &item);
			message->FindInt32("index", &index);
			if(!strcmp(item->Menu()->Name(),"SourceMenu0")){
				midi->ControlChange(15, 22, index);
			}
			else{
				midi->ControlChange(16, 22, index);
			}
			break;
		}
		case MENU_MODE_CHANGED:
		{
			MenuField *menufield;
			message->FindInt32("index", &index);
			midi->SetEffectMode(4-index);
			message->FindPointer("MenuHide", (void **) &menufield);
			menufield->Hide();
			message->FindPointer("MenuShow", (void **) &menufield);
			menufield->Menu()->ItemAt(0)->SetMarked(true);
			menufield->Show();
			for(int i=0;i<4;i++){
				sprintf(str,"Effect%d",i);
				((BButton*)FindView(str))->SetLabel("effect");
				sprintf(str,"Mute%d",i);
				((BButton*)FindView(str))->SetLabel("mute");
				sprintf(str,"Solo%d",i);
				((BButton*)FindView(str))->SetLabel("solo");
			}
			((MenuField*)FindView("Input"))->Menu()->ItemAt(0)->SetMarked(true);
			//EffectWindow 変更のメッセージを App に送る
			BMessenger messenger(APP_SIGNATURE);
			BMessage AppMessage(EFFECT_WINDOW_CHANGE);
			AppMessage.AddBool("window", false);
			messenger.SendMessage(&AppMessage);
			break;
		}
		case MENU_EFFECT_CHANGED:
		{
			int8 msb,lsb;
			int32 effectindex;
			message->FindInt8("MSB", &msb);
			message->FindInt8("LSB", &lsb);
			midi->SetEffect88Pro(msb,lsb);
			//EffectWindow 変更のメッセージを App に送る
			//まずウィンドウを消す
			BMessenger messenger(APP_SIGNATURE);
			BMessage AppMessage(EFFECT_WINDOW_CHANGE);
			AppMessage.AddBool("window", false);
			messenger.SendMessage(&AppMessage);
			//新しくウィンドウを作り直す
			BMenu *menu = ((MenuField*)FindView("Mode"))->Menu();
			BMenuItem *menuitem;
			message->FindPointer("source", (void **) &menuitem);
			AppMessage.AddInt32("effectmode",menu->IndexOf(menu->FindMarked()));
			message->FindInt32("index",&effectindex);
			AppMessage.AddInt32("effectindex",effectindex);
			AppMessage.AddString("effectname",menuitem->Label());
			AppMessage.ReplaceBool("window", true);
			messenger.SendMessage(&AppMessage);
			break;
		}
		case BUTTON_SOLO_CHANGED:
		{
			message->FindPointer("source", (void **) &button);
			strcpy(name,button->Name());
			for(i=0;i<16;i++){
				sprintf(str,"Solo%d",i);
				if(!strcmp(str,name)){
					if(strcmp(button->Label(),"solo")){
						button->SetLabel("solo");
						midi->ControlChange(i+1,19,false);
					}
					else{
						button->SetLabel("SOLO");
						midi->ControlChange(i+1,19,true);
						for(int j=0;j<4;j++){
							if(j!=i){
								sprintf(str,"Solo%d",j);
								((BButton*)FindView(str))->SetLabel("solo");
							}
						}
					}
					break;
				}
			}
			break;
		}
		case BUTTON_MUTE_CHANGED:
		{
			message->FindPointer("source", (void **) &button);
			strcpy(name,button->Name());
			for(i=0;i<16;i++){
				sprintf(str,"Mute%d",i);
				if(!strcmp(str,name)){
					if(strcmp(button->Label(),"mute")){
						button->SetLabel("mute");
						midi->ControlChange(i+1,18,false);
					}
					else{
						button->SetLabel("MUTE");
						midi->ControlChange(i+1,18,true);
					}
					break;
				}
			}
			break;
		}
		case BUTTON_EFFECT_CHANGED:
		{
			message->FindPointer("source", (void **) &button);
			strcpy(name,button->Name());
			for(i=0;i<16;i++){
				sprintf(str,"Effect%d",i);
				if(!strcmp(str,name)){
					if(strcmp(button->Label(),"effect")){
						button->SetLabel("effect");
						midi->ControlChange(i+1,23,false);
					}
					else{
						button->SetLabel("EFFECT");
						midi->ControlChange(i+1,23,true);
						
						//EffectWindow 変更のメッセージを App に送る
						BMenu *menu;
						BMenuField *menufield;
						BMessenger messenger(APP_SIGNATURE);
						BMessage AppMessage(EFFECT_WINDOW_CHANGE);
						menu = ((MenuField*)FindView("Mode"))->Menu();
						AppMessage.AddInt32("effectmode",menu->IndexOf(menu->FindMarked()));
						if(menu->IndexOf(menu->FindMarked())){
							message->FindPointer("compacteffect", (void **) &menufield);
						}
						else{
							message->FindPointer("fulleffect", (void **) &menufield);
						}
						menu = menufield->Menu();
						AppMessage.AddInt32("effectindex",menu->IndexOf(menu->FindMarked()));
						AppMessage.AddString("effectname",menu->FindMarked()->Label());
						AppMessage.AddBool("window", true);
						messenger.SendMessage(&AppMessage);
						
						//他のボタンをオフ
						for(int j=0;j<4;j++){
							if(j!=i){
								sprintf(str,"Effect%d",j);
								((BButton*)FindView(str))->SetLabel("effect");
							}
						}
					}
					break;
				}
			}
			break;
		}
		default:
		{
			BWindow::MessageReceived(message);
			break;
		}
	}
}

//ウィンドウを閉じるメッセージが来たら設定を保存してアプリ終了メッセージを送出
bool Window::QuitRequested(){
	Prefs->SetRect("main_window_rect", Frame());
	
	BMenu *menu;
	menu = ((MenuField*)FindView("Mode"))->Menu();
	Prefs->SetBool("effect_mode", (bool)(menu->IndexOf(menu->FindMarked())));
	
	menu = ((MenuField*)FindView("Input"))->Menu();
	Prefs->SetInt8("input_mode", (int8)(menu->IndexOf(menu->FindMarked())));
	
	menu = (BMenu*)(((MenuBar*)FindView("MenuBar"))->SubmenuAt(0));
	Prefs->SetString("port", menu->FindMarked()->Label());

	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

bool EffectWindow::QuitRequested(){
	Prefs->SetRect("effect_window_rect", Frame());
	return true;
}

//ウィンドウクラスのコンストラクタ
Window::Window(BRect frame)
	: BWindow(
		frame,
		"ua100panel",
		B_TITLED_WINDOW,
		B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS
	){
	BRect rect;
	MenuBar *menubar;
	View *view;
	rect=Bounds();
	menubar = new MenuBar(rect);
	AddChild(menubar);
	rect.top = menubar->Bounds().bottom+1;
	view = new View(rect);
	AddChild(view);
	Show();
}

void EffectView::Draw(BRect updateRect){
	if(BGBitmap!=NULL)DrawBitmap(BGBitmap);
}

//エフェクトビュークラスのコンストラクタ
EffectView::EffectView(BRect frame, EffectWindow *window, int32 effectmode, int32 effectindex)
	:BView(frame,
		NULL,
		B_FOLLOW_ALL_SIDES,
		B_WILL_DRAW
		){
	//Load bitmap
	char filename[256];
	if(effectmode){
		sprintf(filename,"%s/block/4_EFX%02d.bmp",MyPath.Path(),(int)effectindex);
	}
	else{
		sprintf(filename,"%s/block/speefx%02d.bmp",MyPath.Path(),(int)effectindex+1);
	}
	BGBitmap = BTranslationUtils::GetBitmapFile(filename);
	SetViewColor(EFFECT_VIEW_COLOR);
	
	EfxData *efxdata;
	if(effectmode){
		efxdata=Efx88Data;
	}
	else{
		efxdata=EfxSpData;
	}
	
	if(!(efxdata->InitCheck()))return;
	
	//Sliders
	int32 num,tmp;
	int min,max,defvalue=100,type;
	float DiagramSize = DIAGRAM_SIZE*(BGBitmap!=NULL);
	char section[16],key[16],*str,*pname;
	BMessage *message;
	BRect frame;
	frame=Bounds();
	frame.top+=DiagramSize+2;
	frame.left+=2;
	frame.bottom=frame.top+SECTION_SIZE_PARAMETER_Y+2;
	frame.right = frame.left+SECTION_SIZE_PARAMETER_X;
	
	sprintf(section,"PNAME%d_",(int)effectindex+1);
	strcpy(key,section);
	strcat(key,"NUM");
	efxdata->FindString(key,(const char **)&str);
	sscanf(str,"%d",(int*)&num);

	window->ResizeTo((SECTION_SIZE_PARAMETER_X+3)*4,
		(SECTION_SIZE_PARAMETER_Y+3)*(num*(num<EFFECT_COLUM)+EFFECT_COLUM*(num>=EFFECT_COLUM))+DiagramSize);
	ResizeTo(window->Bounds().Width(),window->Bounds().Height());
	for(int i=0;i<(int)num;i++){
		message = new BMessage(EFFECT_PARAMETER_CHANGED);

		sprintf(key, "%s%d", section, i+1);
		efxdata->FindString(key,(const char **)&pname);

		sprintf(key, "%sV%d", section, i+1);
		efxdata->FindString(key,(const char **)&str);
		sscanf(str,"%d",&type);

		sprintf(key,"%sD%d",section,i+1);
		efxdata->FindString(key,(const char **)&str);
		sscanf(str,"%d",&defvalue);

		sprintf(key,"%sL%d",section,i+1);
		efxdata->FindString(key,(const char **)&str);
		sscanf(str,"%X",(uint*)&tmp);
		
		message->AddInt8("address", (int8)tmp);
		
		//タイプによって minimum,maximum を変更
		sprintf(key,"%d_min",type);
		if(Limits->FindString(key,(const char **)&str)==B_OK)
			sscanf(str,"%X",(uint*)&min);
		else
			min=0;

		sprintf(key,"%d_max",type);
		if(Limits->FindString(key,(const char **)&str)==B_OK)
			sscanf(str,"%X",(uint*)&max);
		else
			max=127;

		//スライダー作成
		str = new char[16];
		sprintf(str,"Slider%d",i);
		AddChild(new Slider(frame,str,message,EFFECT_SLIDER_COLOR,
			min,max,defvalue,pname,type));
		frame.left-=2;
		frame.top-=2;
		frame.right+=2;
//		frame.bottom+=2;
		AddChild(new BBox(frame));
		frame.left+=2;
		frame.top+=2;
		frame.right-=2;
//		frame.bottom-=2;
		
		frame.top+=SECTION_SIZE_PARAMETER_Y+3;
		frame.bottom+=SECTION_SIZE_PARAMETER_Y+3;
		if(i%EFFECT_COLUM==EFFECT_COLUM-1){
		frame.left+=SECTION_SIZE_PARAMETER_X+3;
		frame.right+=SECTION_SIZE_PARAMETER_X+3;
		frame.top=Bounds().top+DiagramSize+2;
		frame.bottom=frame.top+SECTION_SIZE_PARAMETER_Y+2;
		}
	}
}


//エフェクトウィンドウクラスのコンストラクタ
EffectWindow::EffectWindow(BRect frame, char *name, int32 effectmode, int32 effectindex)
	: BWindow(
		frame,
		name,
		B_TITLED_WINDOW,
		B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS|B_NOT_CLOSABLE
	){
	BRect rect;
	rect=Bounds();
	EffectView *view;
	view = new EffectView(rect, this, effectmode, effectindex);
	AddChild(view);
	Show();
}

//エフェクトウィンドウのメッセージループ
void EffectWindow::MessageReceived(BMessage *message){
	switch(message->what){
		case EFFECT_PARAMETER_CHANGED:
		{
			int8 address;
			int value;
			Slider *slider;
			message->FindInt8("address", &address);
			message->FindPointer("source", (void**)&slider);
			switch(slider->Type()){
				case 3:
				case 6:
				case 18:
				case 32:
				case 44:
					value = slider->Value()*8;
					break;
				case 39:
					if(slider->Value() == 0)value = 0;
					else value = 3 + slider->Value()*4;
					break;
				default:
					value = slider->Value();
					break;
			}
			midi->SetEffectParameter88Pro(value, address);
			break;
		}
		default:
		{
			BWindow::MessageReceived(message);
			break;
		}
	}
}


//アプリケーションクラスのメッセージループ
void App::MessageReceived(BMessage *message){
	switch(message->what){
		case EFFECT_WINDOW_CHANGE:
		{
			bool bWindow;
			message->FindBool("window", &bWindow);
			if(bWindow && !EffectWindowCount){
				int32 effectmode, effectindex;
				char *effectname;
				BRect frame;
				if(Prefs->FindRect("effect_window_rect", &frame) != B_OK){
					frame.Set(460,100,600,400);
				}
				message->FindInt32("effectmode", &effectmode);
				message->FindInt32("effectindex", &effectindex);
				message->FindString("effectname", (const char**) &effectname);
				aEffectWindow = new EffectWindow(frame, effectname, effectmode, effectindex);
				EffectWindowCount=true;
			}
			if(!bWindow && EffectWindowCount){
				aEffectWindow->PostMessage(B_QUIT_REQUESTED);
				EffectWindowCount=false;
			}
		}
		default:
		{
			BApplication::MessageReceived(message);
			break;
		}
	}
}

//アプリケーションクラスのコンストラクタ
App::App() : BApplication(APP_SIGNATURE) {
	midi= new MidiPort();
	app_info info;
	BEntry *entry, dir_entry;
	GetAppInfo(&info);
	entry = new BEntry(&(info.ref));
	entry->GetParent(&dir_entry);
	dir_entry.GetPath(&MyPath);
	Efx88Data = new EfxData(new BPath(MyPath),"effsc88.dat");
	EfxSpData = new EfxData(new BPath(MyPath),"effspe.dat");
	SourceData = new EfxData(new BPath(MyPath),"source.dat");
	Limits = new EfxData(new BPath(MyPath),"limits.dat");
	
	Prefs = new TPreferences("ua100panel.cfg");
	int32 count;
	if (Prefs->FindInt32("use_count", &count) != B_OK) {
		count = 0;
	}
	Prefs->SetInt64("last_used", real_time_clock());
	Prefs->SetInt32("use_count", ++count);

	//ウィンドウ作成
	BRect aFrame;
	if (Prefs->FindRect("main_window_rect", &aFrame) != B_OK) {
		aFrame.Set(100, 80, 450, 410);
	}
	Window* aWindow = NULL;
	aWindow = new Window(aFrame);
	EffectWindowCount=0;
}

//メイン関数、アプリケーションオブジェクトの作成と実行のみ
int main(int argc, char *argv[]){
	new App();
	be_app->Run();
	delete Prefs;
}
