class MidiPort : public BMidiPort{
	public:
		virtual void SetEffectMode(int8 mode=4);
		virtual void SetEffect88Pro(int8 data1, int8 data2);
		virtual void SetEffectParameter88Pro(int8 data1, int address);
};

class EfxData : public BMessage{
	public:
		EfxData(BPath *path, const char *path);
		bool InitCheck(void);
	private:
		bool status;
};

class TPreferences : public BMessage {
	public:
				TPreferences(char *filename);
				~TPreferences();
		status_t InitCheck(void);
		
		status_t SetBool(const char *name, bool b);
		status_t SetInt8(const char *name, int8 i);
		status_t SetInt16(const char *name, int16 i);
		status_t SetInt32(const char *name, int32 i);
		status_t SetInt64(const char *name, int64 i);
		status_t SetFloat(const char *name, float f);
		status_t SetDouble(const char *name, double d);
		status_t SetString(const char *name, const char *string);
		status_t SetPoint(const char *name, BPoint p);
		status_t SetRect(const char *name, BRect r);
		status_t SetMessage(const char *name, const BMessage *message);
		status_t SetFlat(const char *name, const BFlattenable *obj);
	
	private:
		
		BPath path;
		status_t status;
};

inline status_t TPreferences::InitCheck(void) {
	return status;
}

//ファイルからメニュー項目を読み出すメニュー
class Menu : public BMenu{
	public:
		Menu(const char *label, EfxData *efxdata, char *section,
			const uint32 messageid=0);
};

class MenuBar : public BMenuBar{
	public:
		MenuBar(BRect frame);
};

class MenuField : public BMenuField{
	public:
		MenuField(BRect frame, const char *name, BMenu *menu);
};

class Slider : public BSlider{
	public:
		Slider(BRect frame, char *name, BMessage *message, rgb_color color,
			uint16 min=0, uint16 max=127, uint16 defvalue = 100,
			const char *label = "", int typetmp = 0);
		virtual char *UpdateText(void) const;
		int Type(void);
	private:
		int type;
};

class View : public BView{
	public:
		View(BRect frame);
		virtual void Draw(BRect updateRect);
};

//EffectWindow
class EffectWindow : public BWindow{
	public:
		EffectWindow(BRect frame, char *name, int32 effectmode, int32 effectindex);
		virtual void MessageReceived(BMessage *message);
		virtual bool QuitRequested();
};

class EffectView : public BView{
	public:
		EffectView(BRect frame, EffectWindow *window, int32 effectmode, int32 effectindex);
		virtual void Draw(BRect updateRect);
	private:
		BBitmap *BGBitmap;
};

//MainWindow
class Window : public BWindow{
	public:
		Window(BRect frame);
		virtual bool QuitRequested();
		virtual void MessageReceived(BMessage *message);
};

class App : public BApplication{
	public:
		App();
		virtual void MessageReceived(BMessage *message);
	private:
		Window *aWindow;
		EffectWindow *aEffectWindow;
		bool EffectWindowCount;
};

//メッセージの宣言
const uint32 INPUT_SLIDER_CHANGED = 'ISch';
const uint32 OUTPUT_SLIDER_CHANGED = 'OSch';
const uint32 REC_SLIDER_CHANGED = 'RSch';
const uint32 MENU_PORT_CHANGED = 'MPch';
const uint32 MENU_INPUT_CHANGED = 'MIch';
const uint32 MENU_SOURCE_CHANGED = 'MSch';
const uint32 MENU_MODE_CHANGED = 'MMch';
const uint32 MENU_EFFECT_CHANGED = 'MEch';
const uint32 BUTTON_SOLO_CHANGED = 'BSch';
const uint32 BUTTON_MUTE_CHANGED = 'BMch';
const uint32 BUTTON_EFFECT_CHANGED = 'BEch';
const uint32 EFFECT_WINDOW_CHANGE = 'EWch';
const uint32 EFFECT_PARAMETER_CHANGED = 'EPch';
const uint32 EFFECT_DATA = 'EDch';

//定数
const char *DEF_PORT_NAME = "/dev/midi/ua100/3";
const char *APP_SIGNATURE = "application/x-vnd.ua100panel";
const char *SliderLabel[16]=
	{
		"INPUT",
		"MIC2",
		"WAVE1",
		"WAVE2",
		"SysRET",
		"SysSUB",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		"REC",
		"MASTER"
	};
const float SECTION_SIZE_INPUT = 50;
const float SECTION_SIZE_OUTPUT = 65;
const float SECTION_SIZE_PARAMETER_X = 110;
const float SECTION_SIZE_PARAMETER_Y = 50;
const float DIAGRAM_SIZE = 121;
const float SECTION_SPACE = 5;
const int EFFECT_COLUM = 6;

	
rgb_color MAIN_VIEW_COLOR = {191,191,255,255};
rgb_color MAIN_SLIDER_COLOR = {127,255,127,255};
rgb_color EFFECT_VIEW_COLOR = {127,191,127,255};
rgb_color EFFECT_SLIDER_COLOR = {191,127,191,255};
/*			
			char str[8];
			sprintf(str,"%d",value);
			BAlert *alert;
			alert = new BAlert("info",str,"OK");
			alert->Go();
*/			
